"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const promisify_1 = __importDefault(require("../../lib/promisify"));
const glob_1 = __importDefault(require("glob"));
const constants_1 = require("next-server/constants");
const glob = promisify_1.default(glob_1.default);
async function getPages(dir, { nextPagesDir, dev, buildId, isServer, pageExtensions }) {
    const pageFiles = await getPagePaths(dir, { dev, isServer, pageExtensions });
    return getPageEntries(pageFiles, { nextPagesDir, buildId, isServer, pageExtensions });
}
exports.getPages = getPages;
async function getPagePaths(dir, { dev, isServer, pageExtensions }) {
    let pages;
    if (dev) {
        // In development we only compile _document.js, _error.js and _app.js when starting, since they're always needed. All other pages are compiled with on demand entries
        pages = await glob(isServer ? `pages/+(_document|_app|_error).+(${pageExtensions})` : `pages/+(_app|_error).+(${pageExtensions})`, { cwd: dir });
    }
    else {
        // In production get all pages from the pages directory
        pages = await glob(isServer ? `pages/**/*.+(${pageExtensions})` : `pages/**/!(_document)*.+(${pageExtensions})`, { cwd: dir });
    }
    return pages;
}
exports.getPagePaths = getPagePaths;
// Convert page path into single entry
function createEntry(filePath, { buildId = '', name, pageExtensions } = {}) {
    const parsedPath = path_1.default.parse(filePath);
    let entryName = name || filePath;
    // This makes sure we compile `pages/blog/index.js` to `pages/blog.js`.
    // Excludes `pages/index.js` from this rule since we do want `/` to route to `pages/index.js`
    if (parsedPath.dir !== 'pages' && parsedPath.name === 'index') {
        entryName = `${parsedPath.dir}.js`;
    }
    // Makes sure supported extensions are stripped off. The outputted file should always be `.js`
    if (pageExtensions) {
        entryName = entryName.replace(new RegExp(`\\.+(${pageExtensions})$`), '.js');
    }
    return {
        name: path_1.default.join(constants_1.CLIENT_STATIC_FILES_PATH, buildId, entryName),
        files: [parsedPath.root ? filePath : `./${filePath}`] // The entry always has to be an array.
    };
}
exports.createEntry = createEntry;
// Convert page paths into entries
function getPageEntries(pagePaths, { nextPagesDir, buildId, isServer = false, pageExtensions } = {}) {
    const entries = {};
    for (const filePath of pagePaths) {
        const entry = createEntry(filePath, { pageExtensions, buildId });
        entries[entry.name] = entry.files;
    }
    const appPagePath = path_1.default.join(nextPagesDir, '_app.js');
    const appPageEntry = createEntry(appPagePath, { buildId, name: 'pages/_app.js' }); // default app.js
    if (!entries[appPageEntry.name]) {
        entries[appPageEntry.name] = appPageEntry.files;
    }
    const errorPagePath = path_1.default.join(nextPagesDir, '_error.js');
    const errorPageEntry = createEntry(errorPagePath, { buildId, name: 'pages/_error.js' }); // default error.js
    if (!entries[errorPageEntry.name]) {
        entries[errorPageEntry.name] = errorPageEntry.files;
    }
    if (isServer) {
        const documentPagePath = path_1.default.join(nextPagesDir, '_document.js');
        const documentPageEntry = createEntry(documentPagePath, { buildId, name: 'pages/_document.js' }); // default _document.js
        if (!entries[documentPageEntry.name]) {
            entries[documentPageEntry.name] = documentPageEntry.files;
        }
    }
    return entries;
}
exports.getPageEntries = getPageEntries;
